/**
 * apeman task to run browserify
 * @function apemanTaskBrowserify
 * @param {object} [options] - Optional settings.
 * @returns {function} - Defined task function.
 */

'use strict'

const argx = require('argx')
const execcli = require('execcli')
const co = require('co')
const expandglob = require('expandglob')
const listExternals = require('./list_externals')

/** @lends define */
function define (src, options) {
  let args = argx(arguments)
  options = args.pop('object') || {}
  src = args.remain()

  /**
   * Defined task.
   * @function task
   * @param {object} context - Apeman task context.
   * @returns {Promise}
   */
  function task (context) {
    let logger = context.logger
    return co(function * () {
      let filenames = src
        .map((src) => define.globSync(src))
        .reduce((a, b) => [].concat(a, b), [])
      logger.trace(filenames)
      logger.trace(options)
      yield define.browserify([].concat(filenames).concat(options), {})
    })
  }

  Object.assign(task, {
    // Description of this task.
    $desc: 'Execute browserify command.'
  })

  return task
}

define.external = function (src, options) {
  let args = argx(arguments)
  options = args.pop('object') || {}
  src = args.remain()

  /**
   * Defined task.
   * @function task
   * @param {object} context - Apeman task context.
   * @returns {Promise}
   */
  function task (context) {
    let logger = context.logger
    let filenames = src
      .map((src) => define.globSync(src))
      .reduce((a, b) => [].concat(a, b), [])
    return co(function * () {
      let names = yield listExternals(filenames)
      logger.debug(names)
      return names
    })
  }

  Object.assign(task, {
    // Description of this task.
    $desc: 'List external modules.'
  })
  return task
}

define.browserify = function (args, options) {
  let execOptions = Object.assign({
    notfound: 'try `npm install browserify -g`',
    search: [ process.cwd(), __dirname ]
  }, options)
  return new Promise((resolve, reject) => {
    execcli.npmBin('browserify', args, execOptions, (err) =>
      err ? reject(err) : resolve()
    )
  })
}

define.globSync = function (src) {
  if (/\*/.test(src)) {
    return expandglob.sync(src)
  }
  return src
}

module.exports = define
