/**
 * List external modules.
 * @function externals
 */
'use strict'

const cp = require('child_process')
const path = require('path')
const co = require('co')

/** @lends externals */
function externals (src) {
  return co(function * () {
    yield new Promise((resolve, reject) => {
      cp.exec(
        `browserify --list --fast ${[].concat(src).join(' ')}`,
        (err, stdOut) => {
          let modules = !err && String(stdOut).split(/\n/g)
              .map((filename) => _pkg(filename))
              .filter((filename) => !!filename)
              .map((filename) => require(filename).name)
              .sort((a, b) => a.localeCompare(b))
              .filter((v, i, array) => array.indexOf(v) === i)
          err ? reject(err) : resolve(modules)
        }
      )
    })
  })
}

// Findup package.json
function _pkg (filename) {
  let dirname = path.dirname(filename)
  try {
    return require.resolve(path.join(dirname, 'package.json'))
  } catch (e) {
    if (filename === dirname) {
      return null
    } else {
      return _pkg(dirname)
    }
  }
}

module.exports = externals
