/**
 * Test case for define.
 * Runs with mocha.
 */
'use strict'

const define = require('../lib/define.js')
const mkdirp = require('mkdirp')
const assert = require('assert')
const co = require('co')

describe('define', () => {
  let tmpDir = `${__dirname}/../tmp`

  function noop () {
  }

  before(() => co(function * () {
    mkdirp.sync(tmpDir)
  }))

  after(() => co(function * () {
  }))

  it('Define', () => co(function * () {
    let task = define(
      `${__dirname}/../doc/mocks/mock-entry01.js`,
      `${__dirname}/../doc/mocks/mock-entry02.js`,
      {
        outfile: `${tmpDir}/testing-bundle.js`
      }
    )
    assert.ok(task)
    let context = {
      verbose: false,
      logger: {
        info: noop,
        debug: noop,
        trace: noop
      }
    }
    yield task(context)
  }))

  it('List externals', () => co(function * () {
    let task = define.external(
      `${__dirname}/../doc/mocks/mock-entry01.js`,
      `${__dirname}/../doc/mocks/mock-entry02.js`
    )
    assert.ok(task)
    let context = {
      verbose: false,
      logger: {
        info: noop,
        debug: console.log,
        trace: noop
      }
    }
    yield task(context)
  }))
})

/* global describe, before, after, it */
